from PySide6.QtWidgets import QVBoxLayout, QWidget
from matplotlib import pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas, \
    NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure

plt.rcParams['font.sans-serif'] = ['SimHei']  # 用来正常显示中文标签
plt.rcParams['axes.unicode_minus'] = False  # 用来正常显示负号


class MplCanvas(QWidget):
    def __init__(self, enable_toolbar: bool = True, parent=None):
        super(MplCanvas, self).__init__(parent)

        self.canvas = FigureCanvas(Figure(figsize=(5, 4), constrained_layout=True))
        self.ax = self.canvas.figure.subplots()

        self.vertical_layout = QVBoxLayout()
        if enable_toolbar:
            self.toolbar = NavigationToolbar(self.canvas, self)
            self.vertical_layout.addWidget(self.toolbar)
        self.vertical_layout.addWidget(self.canvas)
        self.setLayout(self.vertical_layout)

    def clear_canvas(self):
        self.canvas.figure.clear()
        self.ax = self.canvas.figure.subplots()
        self.canvas.draw_idle()

    def plot_lines(self, data: list[dict],
                   x_label: str = '',
                   y_label: str = '',
                   title: str = ''):

        enable_legend = False
        for item in data:
            if item.get('label'):
                enable_legend = True
                self.ax.plot(item['x'], item['y'], marker='o', markersize=3, label=item['label'])
            else:
                self.ax.plot(item['x'], item['y'], marker='o', markersize=3)

        # 设置轴标签和标题
        self.ax.set_xlabel(x_label)
        self.ax.set_ylabel(y_label)
        self.ax.set_title(title)
        if enable_legend:
            self.ax.legend(loc="upper left")
        self.canvas.draw_idle()



from PySide6.QtCore import QObject, Signal


def create_signal(signal_type):
    """根据传入类型，创建相应的信号类。"""
    class CustomSignal(QObject):
        signal = Signal(signal_type)

    return CustomSignal()


class ResponseProgressSignal:
    """需要某个功能类继承它，并连接response_progress_signal方法到某一信号。
    然后根据需要实现各种progress_xxx方法，即可根据不同状态自动调用相应的处理方法。"""

    def response_progress_signal(self, state: tuple):
        state_flag, data = state
        if state_flag == ProgressState.flag_start:
            self.progress_start(data)
        elif state_flag == ProgressState.flag_doing:
            self.progress_doing(data)
        elif state_flag == ProgressState.flag_success:
            self.progress_success(data)
        elif state_flag == ProgressState.flag_failed:
            self.progress_failed(data)
        elif state_flag == ProgressState.flag_end:
            self.progress_end(data)
        elif state_flag == ProgressState.flag_info:
            self.progress_info(data)
        elif state_flag == ProgressState.flag_other1:
            self.progress_other1(data)
        elif state_flag == ProgressState.flag_other2:
            self.progress_other2(data)
        elif state_flag == ProgressState.flag_other3:
            self.progress_other3(data)
        else:
            raise ValueError(f'传递的信号数据错误。应该是 tuple ，但得到的是 {type(state)}')

    def progress_start(self, data: list = None):
        pass

    def progress_doing(self, data: list = None):
        pass

    def progress_success(self, data: list = None):
        pass

    def progress_failed(self, data: list = None):
        pass

    def progress_end(self, data: list = None):
        pass

    def progress_info(self, data: list = None):
        pass

    def progress_other1(self, data: list = None):
        pass

    def progress_other2(self, data: list = None):
        pass

    def progress_other3(self, data: list = None):
        pass


class ProgressState:
    """发射信号时应该使用此类来传输数据。

    Example:
        # 具体数据应使用list包裹，这利于处理复杂形态的数据。
        some_signal.emit(ProgressState.start(['处理开始']))

    """
    flag_start = 0
    flag_doing = 1
    flag_end = 2
    flag_success = 3
    flag_failed = 4
    flag_info = 5
    # 预留的其他状态量
    flag_other1 = 6
    flag_other2 = 7
    flag_other3 = 8

    @staticmethod
    def start(data: list = None) -> tuple:
        return ProgressState.flag_start, data

    @staticmethod
    def doing(data: list = None) -> tuple:
        return ProgressState.flag_doing, data

    @staticmethod
    def end(data: list = None) -> tuple:
        return ProgressState.flag_end, data

    @staticmethod
    def success(data: list = None) -> tuple:
        return ProgressState.flag_success, data

    @staticmethod
    def failed(data: list = None) -> tuple:
        return ProgressState.flag_failed, data

    @staticmethod
    def info(data: list = None) -> tuple:
        return ProgressState.flag_info, data

    @staticmethod
    def other1(data: list = None) -> tuple:
        return ProgressState.flag_other1, data

    @staticmethod
    def other2(data: list = None) -> tuple:
        return ProgressState.flag_other2, data

    @staticmethod
    def other3(data: list = None) -> tuple:
        return ProgressState.flag_other3, data



# -*- coding:utf-8 -*-
from PySide6 import QtGui, QtCore
from PySide6.QtCore import QItemSelectionModel, Qt
from PySide6.QtGui import QStandardItem
from PySide6.QtWidgets import QTableView

from qtlink.util import create_signal


class TableController:
    def __init__(self, tableview: QTableView,
                 highlight_hover_row: bool = False):
        self.tableview = tableview
        self.table_columns = None
        self.raw_data = None
        self.column_widths = None
        self.first_load = True
        self.model = QtGui.QStandardItemModel()
        self.signal_click_row = create_signal(dict)
        # 连接信号
        self.tableview.clicked.connect(self.on_table_clicked)
        # 禁止上下文菜单
        scrollBar = self.tableview.verticalScrollBar()
        scrollBar.setContextMenuPolicy(Qt.NoContextMenu)
        scrollBar = self.tableview.horizontalScrollBar()
        scrollBar.setContextMenuPolicy(Qt.NoContextMenu)
        # 启用高亮一整行效果
        if highlight_hover_row:
            self.tableview.setMouseTracking(True)
            self.hover_delegate = HoverDelegate(tableview=tableview)
            self.tableview.entered.connect(self.hover_delegate.on_entered)
        else:
            self.hover_delegate = None

    def update_model_data(self, model_data: list[dict], hide_columns: list[str] = None):
        if hide_columns is None:
            hide_columns = []
        if not self.first_load:
            # 保存当前列宽
            self.save_current_column_widths()
        self.first_load = False

        self.model.clear()
        self.raw_data = model_data
        for data in model_data:
            items = [QStandardItem(str(data.get(column, None)) if data.get(column, None) is not None else '')
                     for column in self.table_columns
                     if column not in hide_columns]
            self.model.appendRow(items)
        self.update_table_columns(self.table_columns)

    def update_table_columns(self, table_columns: list[str]):
        self.table_columns = table_columns
        self.model.setHorizontalHeaderLabels(table_columns)
        self.tableview.setModel(self.model)

        if self.column_widths is not None:
            for item in self.column_widths:
                index, width = item
                self.tableview.setColumnWidth(index, width)

    def clear_model_data(self):
        self.update_model_data(model_data=[])

    def save_current_column_widths(self):
        if self.column_widths is None:
            return
        for index in range(self.model.columnCount()):
            width = self.tableview.columnWidth(index)
            # 确保保存的列宽信息与列的数量相匹配
            if index < len(self.column_widths):
                self.column_widths[index] = (index, width)
            else:
                self.column_widths.append((index, width))

    def on_table_clicked(self, index):
        # index是QModelIndex类型，可以用来检索数据
        row = index.row()  # 获取行号
        # 获取该行的所有列的数据
        rowData = self.raw_data[row]
        self.signal_click_row.signal.emit(rowData)

    def set_column_width(self, column_widths: list[tuple] = None):
        self.column_widths = column_widths

    def set_select_row(self, index: int):
        if len(self.raw_data) - 1 >= index:
            first_index = self.model.index(index, 0)
            self.tableview.selectionModel().select(first_index,
                                                   QItemSelectionModel.Select | QItemSelectionModel.Rows)

    def set_not_select_row(self, index: int):
        if len(self.raw_data) - 1 >= index:
            first_index = self.model.index(index, 0)
            self.tableview.selectionModel().select(first_index,
                                                   QItemSelectionModel.Deselect | QItemSelectionModel.Rows)

    def __del__(self):
        if self.hover_delegate and self.tableview.viewport():
            self.tableview.viewport().removeEventFilter(self.hover_delegate)


class HoverDelegate(QtCore.QObject):
    def __init__(self, tableview):
        super().__init__(tableview)
        self.tableview = tableview
        self.row_under_mouse = -1
        self.hover_color = QtGui.QColor(240, 240, 240)
        # 安装事件过滤器到表格视图的视口上
        self.tableview.viewport().installEventFilter(self)
        self.is_hovering = False

    def on_entered(self, index):
        self.is_hovering = True
        hover_row = index.row()
        # 检查当前行是否被选中
        is_row_selected = self.tableview.selectionModel().isRowSelected(hover_row, QtCore.QModelIndex())
        if hover_row != self.row_under_mouse:
            # 还原上一行的背景色
            if self.row_under_mouse >= 0:
                self.set_row_background(self.row_under_mouse, self.tableview.palette().base())
            # 重设当前行背景色
            if not is_row_selected:
                self.set_row_background(hover_row, self.hover_color)
                self.row_under_mouse = hover_row
        self.is_hovering = False

    def set_row_background(self, hover_row, color):
        for column in range(self.tableview.model().columnCount()):
            index = self.tableview.model().index(hover_row, column)
            rect = self.tableview.visualRect(index)
            self.tableview.viewport().update(rect)
            self.tableview.model().setData(index, color, Qt.BackgroundRole)

    def clear_hover_style(self):
        if self.row_under_mouse >= 0:
            self.set_row_background(self.row_under_mouse, self.tableview.palette().base())
            self.row_under_mouse = -1
            self.tableview.viewport().update()  # 更新视图以反映背景色的变化

    def eventFilter(self, watched, event):
        if watched and watched == self.tableview.viewport() and event.type() == QtCore.QEvent.Leave:  # noqa
            self.clear_hover_style()
        return super().eventFilter(watched, event)



from PySide6.QtWidgets import QDialog, QTableView, QLabel, QVBoxLayout

from qtlink.table.multiple_select_table_controller import MultipleSelectTableController
from qtlink.table.single_select_table_controller import SingleSelectTableController
from qtlink.table.table_controller import TableController


def show_dialog_use_table(text: str, table_data: list[dict], table_columns, show_or_exec: str = 'exec'):
    dialog = DialogUseTable(text, table_data=table_data, table_columns=table_columns)
    if show_or_exec == 'exec':
        dialog.exec()
    else:
        dialog.show()


class DialogUseTable(QDialog):
    def __init__(self, text: str, table_data: list[dict], table_columns, table_type: str = 'no_select', parent=None):
        super().__init__(parent)
        vLayout = QVBoxLayout()
        label = QLabel(text, self)
        label.setWordWrap(True)
        self.tableview = QTableView(self)
        vLayout.addWidget(label)
        vLayout.addWidget(self.tableview)
        self.setLayout(vLayout)
        if table_type == 'no_select':
            self.table_controller = TableController(tableview=self.tableview)
        elif table_type == 'single_select':
            self.table_controller = SingleSelectTableController(tableview=self.tableview)
        elif table_type == 'multiple_select':
            self.table_controller = MultipleSelectTableController(tableview=self.tableview)
        else:
            self.table_controller = TableController(tableview=self.tableview)
        self.update_table_columns(table_columns)
        self.update_table_data(table_data)

    def update_table_columns(self, table_columns):
        self.table_controller.update_table_columns(table_columns=table_columns)

    def update_table_data(self, table_data):
        self.table_controller.update_model_data(model_data=table_data)



# -*- coding:utf-8 -*-
from PySide6.QtCore import Qt, QRect
from PySide6.QtGui import QStandardItem
from PySide6.QtWidgets import QTableView, QStyleOptionButton, QStyle, QHeaderView

from qtlink.table.table_controller import TableController
from qtlink.util import create_signal


class MultipleSelectTableController(TableController):
    def __init__(self, tableview: QTableView):
        super().__init__(tableview)
        self.model.itemChanged.connect(self.on_item_changed)
        self.signal_select_rows = create_signal(list)

        self.header = CustomHeader(toggle_check_state=self.toggle_check_state, parent=self.tableview)
        self.tableview.setHorizontalHeader(self.header)

        self.always_checked_data = []

    def update_model_data(self, model_data: list[dict],
                          hide_columns: list[str] = None,
                          always_checked_data: list = None):
        # 断开信号连接
        self.model.itemChanged.disconnect(self.on_item_changed)

        if hide_columns is None:
            hide_columns = []
        if not self.first_load:
            # 保存当前列宽
            self.save_current_column_widths()
        self.first_load = False

        self.model.clear()
        self.raw_data = model_data
        self.always_checked_data = always_checked_data if always_checked_data else []
        for data in model_data:
            items = [QStandardItem(str(data.get(column, None)) if data.get(column, None) is not None else '')
                     for column in self.table_columns
                     if column not in hide_columns]
            items[0].setCheckable(True)
            if data in self.always_checked_data:
                items[0].setCheckState(Qt.Checked)
                # 禁用交互
                for item in items:
                    item.setFlags(Qt.ItemIsUserCheckable | Qt.ItemIsSelectable)

            self.model.appendRow(items)
        self.update_table_columns(self.table_columns)
        # 重新连接信号
        self.model.itemChanged.connect(self.on_item_changed)
        self.on_item_changed()

    def toggle_check_state(self, is_checked):
        # 断开信号连接
        self.model.itemChanged.disconnect(self.on_item_changed)

        for row in range(self.model.rowCount()):
            item = self.model.item(row)
            # 重设勾选状态
            if self.raw_data[row] not in self.always_checked_data:
                item.setCheckState(Qt.Checked if is_checked else Qt.Unchecked)

        # 重新连接信号
        self.model.itemChanged.connect(self.on_item_changed)
        self.on_item_changed()

    def get_checked_rows(self):
        checked_rows = []
        for row in range(self.model.rowCount()):
            item = self.model.item(row, 0)  # 勾选框在第一列
            if item.checkState() == Qt.Checked:
                checked_rows.append(self.raw_data[row])
        return checked_rows

    def on_item_changed(self, item: QStandardItem = None):
        if self.hover_delegate is not None and self.hover_delegate.is_hovering:
            return

        checked_rows = self.get_checked_rows()
        self.signal_select_rows.signal.emit(checked_rows)
        checked_rows_num = len(checked_rows)
        if checked_rows_num and checked_rows_num == len(self.raw_data):
            self.set_header_check_state(True)
        else:
            self.set_header_check_state(False)

    def set_header_check_state(self, is_checked: bool):
        self.header.update_check_state(is_checked)


class CustomHeader(QHeaderView):
    def __init__(self, toggle_check_state, orientation=Qt.Horizontal, parent=None):
        super(CustomHeader, self).__init__(orientation, parent)
        self.isChecked = False  # 初始化复选框状态为未选中
        self.toggle_check_state = toggle_check_state

    def update_check_state(self, is_checked):
        self.isChecked = is_checked
        self.updateSection(0)

    def mouseReleaseEvent(self, event):
        # 处理鼠标释放事件，用于切换表头复选框的状态
        index = self.logicalIndexAt(event.position().toPoint())
        if index == 0:
            self.update_check_state(not self.isChecked)  # 切换复选框状态
            self.toggle_check_state(self.isChecked)  # 更新模型中所有项的复选框状态
        else:
            super(CustomHeader, self).mouseReleaseEvent(event)

    def paintSection(self, painter, rect, logicalIndex):
        # 绘制表头的每一部分，用于绘制复选框
        painter.save()
        super(CustomHeader, self).paintSection(painter, rect, logicalIndex)
        painter.restore()
        # 如果是第一列，绘制复选框
        if logicalIndex == 0:
            option = QStyleOptionButton()
            checkbox_width = 10  # 复选框宽度
            checkbox_height = 10  # 复选框高度
            checkbox_x = rect.x() + 3  # 复选框的水平位置
            checkbox_y = rect.y() + (rect.height() - checkbox_height) / 2  # 复选框的垂直位置
            option.rect = QRect(checkbox_x, checkbox_y, checkbox_width, checkbox_height)
            option.state = QStyle.State_Enabled | QStyle.State_Active
            if self.isChecked:
                option.state |= QStyle.State_On  # 设置复选框为选中状态
            else:
                option.state |= QStyle.State_Off  # 设置复选框为未选中状态
            self.style().drawControl(QStyle.CE_CheckBox, option, painter)  # 绘制复选框



# -*- coding:utf-8 -*-
from PySide6.QtCore import Qt
from PySide6.QtGui import QStandardItem
from PySide6.QtWidgets import QTableView

from qtlink.table.table_controller import TableController
from qtlink.util import create_signal


class SingleSelectTableController(TableController):
    def __init__(self, tableview: QTableView):
        super().__init__(tableview)
        self.model.itemChanged.connect(self.on_item_changed)
        self.signal_select_row = create_signal(dict)

    def update_model_data(self, model_data: list[dict], hide_columns: list[str] = None):
        # 断开信号连接
        self.model.itemChanged.disconnect(self.on_item_changed)

        if hide_columns is None:
            hide_columns = []
        if not self.first_load:
            # 保存当前列宽
            self.save_current_column_widths()
        self.first_load = False

        self.model.clear()
        self.raw_data = model_data
        for data in model_data:
            items = [QStandardItem(str(data.get(column, None)) if data.get(column, None) is not None else '')
                     for column in self.table_columns
                     if column not in hide_columns]
            items[0].setCheckable(True)
            self.model.appendRow(items)
        self.update_table_columns(self.table_columns)
        # 重新连接信号
        self.model.itemChanged.connect(self.on_item_changed)
        self.on_item_changed()

    def on_item_changed(self, item: QStandardItem = None):
        if item is None or item.checkState() != Qt.Checked:
            return

        for row in range(self.model.rowCount()):
            if self.model.item(row, 0) != item:
                self.model.item(row, 0).setCheckState(Qt.Unchecked)

        self.signal_select_row.signal.emit(self.raw_data[item.row()])

    def get_checked_row(self):
        for row in range(self.model.rowCount()):
            item = self.model.item(row, 0)  # 勾选框在第一列
            if item.checkState() == Qt.Checked:
                return self.raw_data[row]



from PySide6.QtWidgets import QDialog, QVBoxLayout, QLabel, QPushButton


def show_dialog_info(text: str, enable_btn_confirm: bool = False, show_or_exec: str = 'exec'):
    dialog = DialogInfo(text, enable_btn_confirm)
    if show_or_exec == 'exec':
        dialog.exec()
    else:
        dialog.show()


class DialogInfo(QDialog):
    def __init__(self, text: str, enable_btn_confirm: bool = False):
        super().__init__()
        vLayout = QVBoxLayout()

        label = QLabel(text, self)
        label.setWordWrap(True)
        vLayout.addWidget(label)

        if enable_btn_confirm:
            btn_confirm = QPushButton('确定', self)
            btn_confirm.clicked.connect(self.close)
            vLayout.addWidget(btn_confirm)

        self.setLayout(vLayout)



# -*- coding:utf-8 -*-
import os

from PySide6.QtCore import Qt
from PySide6.QtGui import QCursor
from PySide6.QtWidgets import QWidget, QListWidget, QVBoxLayout, QLabel, QListWidgetItem, \
    QAbstractItemView, QHBoxLayout, QSpacerItem, QSizePolicy

scrollbar_style = """
QScrollBar:vertical {
    border: none;
    background: #F1F1F1;
    width: 12px;
    margin: 0px 0px 0px 0px;
}

QScrollBar::handle:vertical {
    background: #CCCCCC;
    min-height: 20px;
}

QScrollBar:horizontal {
    border: none;
    background: #F1F1F1;
    height: 12px;
    margin: 0px 0px 0px 0px;
}

QScrollBar::handle:horizontal {
    background: #CCCCCC;
    min-width: 20px;
}
"""

list_widget_style = """
QListWidget::item:hover {
    background-color: rgb(240, 240, 240);
    border: none;
    outline: none;
}

QListWidget::item:selected {
    background-color: rgb(220, 220, 220);
    border: none;
    outline: none;
}

QListWidget:focus {
    outline: none;
}

"""


class CustomWidget(QWidget):
    def __init__(self, dict_item):
        super().__init__()
        path = dict_item['path']
        time = dict_item['time']
        self.subtitle = path
        self.setCursor(QCursor(Qt.PointingHandCursor))

        # 创建水平布局
        hLayout = QHBoxLayout()

        # 创建垂直布局并添加标签
        vLayout = QVBoxLayout()
        titleLabel = QLabel(os.path.basename(path), self)
        subtitleLabel = QLabel(path, self)
        subtitleLabel.setStyleSheet("color: rgb(150, 150, 150);")

        vLayout.addWidget(titleLabel)
        vLayout.addWidget(subtitleLabel)

        # 将垂直布局添加到水平布局的左侧
        hLayout.addLayout(vLayout)

        # 添加水平弹簧到水平布局
        spacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        hLayout.addSpacerItem(spacer)

        # 添加一个新的标签到水平布局的右侧
        label_time = QLabel(format_timestamp_in_record(time), self)
        label_time.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
        label_time.setStyleSheet("color: rgb(150, 150, 150);")

        hLayout.addWidget(label_time)

        # 设置CustomWidget的主布局为hLayout
        self.setLayout(hLayout)


class ListWidget(QWidget):
    """只是容器"""

    def __init__(self):
        super().__init__()
        layout = QVBoxLayout(self)
        # 创建QListWidget
        self.listWidget = QListWidget(self)
        layout.addWidget(self.listWidget)
        layout.setContentsMargins(0, 0, 0, 0)  # 设置布局边距为0
        layout.setSpacing(0)  # 设置布局内部组件间的间距为0
        self.setLayout(layout)
        self.setStyleSheet("QWidget { border: none; }")
        # 应用滚动条样式
        self.listWidget.verticalScrollBar().setStyleSheet(scrollbar_style)
        self.listWidget.horizontalScrollBar().setStyleSheet(scrollbar_style)
        self.listWidget.setStyleSheet(list_widget_style)
        # 设置像素级的垂直滚动
        self.listWidget.setVerticalScrollMode(QAbstractItemView.ScrollPerPixel)  # noqa

    def add_widgets(self, widgets: list):
        self.listWidget.clear()
        for widget in widgets:
            item = QListWidgetItem(self.listWidget)
            item.setSizeHint(widget.sizeHint())
            self.listWidget.addItem(item)
            self.listWidget.setItemWidget(item, widget)



from PySide6.QtWidgets import QDialog, QVBoxLayout, QLabel, QPushButton, QHBoxLayout


def show_dialog_choice(text: str, click_btn_confirm, show_or_exec: str = 'exec'):
    dialog = DialogChoice(text, click_btn_confirm)
    if show_or_exec == 'exec':
        dialog.exec()
    else:
        dialog.show()


class DialogChoice(QDialog):
    def __init__(self, text: str, click_btn_confirm):
        super().__init__()
        vLayout = QVBoxLayout()
        self.click_btn_confirm = click_btn_confirm
        label = QLabel(text, self)
        label.setWordWrap(True)
        vLayout.addWidget(label)

        hLayout = QHBoxLayout()
        btn_confirm = QPushButton('确定', self)
        btn_confirm.clicked.connect(self.on_clicked_btn_confirm)
        hLayout.addWidget(btn_confirm)

        btn_cancel = QPushButton('取消', self)
        btn_cancel.clicked.connect(self.close)
        hLayout.addWidget(btn_cancel)
        vLayout.addLayout(hLayout)

        self.setLayout(vLayout)

    def on_clicked_btn_confirm(self):
        self.click_btn_confirm()
        self.close()



# -*- coding:utf-8 -*-
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QWidget, QVBoxLayout, QLabel

from qtlink.util import create_signal


class DropWidget(QWidget):
    def __init__(self, label: str, parent=None):
        super().__init__(parent)
        self.signal_accept_path = create_signal(str)

        self.setAcceptDrops(True)
        self.label = QLabel(label, self)
        self.label.setAlignment(Qt.AlignCenter)
        layout = QVBoxLayout()
        layout.addWidget(self.label)
        self.setLayout(layout)

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.acceptProposedAction()
        else:
            super().dragEnterEvent(event)

    def dropEvent(self, event):
        mimeData = event.mimeData()
        if mimeData.hasUrls():
            # 处理每个URL，这里只取第一个
            url = mimeData.urls()[0]
            # 转换为本地文件路径（去掉 file:/// 前缀）
            path = url.toLocalFile()
            self.signal_accept_path.signal.emit(path)
            event.acceptProposedAction()
        else:
            super().dropEvent(event)
